#!/usr/bin/python
"""
Este modulo contiene el objeto para checkear la conformacion de los archivos
de control escritos en YAML
"""
import datetime
import yaml
import os
import sys

class CheckError(Exception):

    """Excepcion del revisor de archivos de control."""

    def __init__(self, valor):
        self.value = valor

    def __str__(self):
        return self.value


class _Checker(object):

    """Representa una estructura de configuracion."""

    def __init__(self, datos, obligatorias, opcionales=None, una_de_varias=None,
                 si_una_todas=None):
        """Inicializa el objeto."""
        self._estructura = datos
        self._obligatorias = obligatorias
        self._opcionales = opcionales
        self._una_de_varias = una_de_varias
        self._si_una_todas = si_una_todas

    def revisar_obligatorio(self):
        """Revisa que existe una opcion en la estructura.

        Como las estructuras son diccionarios en YAML, es lo mismo que
        revisar que existe el key.

        """
        err_msg = "La estructura '%s' necesita una opcion '%s:'."
        warn_msg = "Se esperaba un valor: %s para la opcion %s, pero esta " + \
                    "tiene un valor %s"
        nombre = 'Sin nombre'
        if self._obligatorias is None:
            return
        # Si la estructura tiene un nombre, utilizarlo para los mensajes
        # de error
        if 'Nombre' in self._estructura:
                nombre = self._estructura['Nombre']

        for nombre_opcion in self._obligatorias:
            if not nombre_opcion in self._estructura:
                raise CheckError(err_msg % (nombre, nombre_opcion))
            else:
                if not isinstance(self._estructura[nombre_opcion],
                                  self._obligatorias[nombre_opcion]):
                    print warn_msg % (self._obligatorias[nombre_opcion],
                                      nombre_opcion,
                                      type(self._estructura[nombre_opcion]))

    def revisar_opcionales(self):
        """Revisa los tipos de variables opcionales

        Como las estructuras son diccionarios en YAML, es lo mismo que
        revisar que existe el key.

        """
        warn_msg = "Se esperaba un valor: %s para la opcion %s, pero esta " + \
                    "tiene un valor %s"
        nombre = 'Sin nombre'
        if self._opcionales is None:
            return
        # Si la estructura tiene un nombre, utilizarlo para los mensajes
        # de error
        if 'Nombre' in self._estructura:
                nombre = self._estructura['Nombre']

        for nombre_opcion in self._opcionales:
            if nombre_opcion in self._estructura:
                if not isinstance(self._estructura[nombre_opcion],
                                  self._opcionales[nombre_opcion]):
                    print warn_msg % (self._opcionales[nombre_opcion],
                                      nombre_opcion,
                                      type(self._estructura[nombre_opcion]))

    def revisar_una_de_varias(self):
        """Revisa que exista al menos una opcion."""
        err_msg = "La estructura '%s' necesita el menos una de las siguientes \
                   opciones  '%s:'"
        if self._una_de_varias is None:
            return
        for grupo in self._una_de_varias:
            for nombre_opcion in grupo:
                # Si la estructura tiene un nombre, utilizarlo para los
                # mensajes de error
                if 'Nombre' in self._estructura:
                    nombre = self._estructura['Nombre']

                if nombre_opcion in self._estructura:
                    return

            raise CheckError(err_msg % (nombre, grupo.keys()))

    def revisar_desconocidas(self):
        """Revisa que todas las opciones seas reconocidas."""
        err_msg = 'No se reconoce la opcion "%s" en "%s"'
        for key in self._estructura:
            if (not key in self._obligatorias) and \
               (not key in self._opcionales):
                for grupo in self._una_de_varias:
                    if not key in grupo:
                        raise CheckError(err_msg % (key, self._estructura['Nombre']))

    def check(self):
        self.revisar_obligatorio()
        self.revisar_opcionales()
        self.revisar_una_de_varias()
        self.revisar_desconocidas()


class TablaChecker(_Checker):

    """Revisor de tablas."""

    def __init__(self, datos):
        """Inicializa el objeto."""
        obligatorios = {'Nombre' : str,
                        'Transformada' : list}

        una_de_varias = [{'Celda': str,
                          'Guardar_en': str}]

        opcionales = {'Expandir_filas': bool,
                     'Expandir_columnas': bool,
                     'Hoja': str,
                     'Hoja_Base': str}
        _Checker.__init__(self, datos, obligatorios, opcionales, una_de_varias)


class DocumentoChecker(_Checker):

    """Revisor de documentos."""

    def __init__(self, datos):
        obligatorias = {'Nombre': str,
                         'Descripcion': str,
                         'Autor': str,
                         'Email': str,
                         'Version_minima': str,
                         'Fecha_edicion': datetime.date,
                         'Plantilla': str,
                         'Archivo_salida': str,
                         'Datos': dict,
                         'Consultas': list}
        opcionales = {'Macros': str}
        _Checker.__init__(self, datos, obligatorias, opcionales)


class ConsultaChecker(_Checker):

    """Revisor de consultas."""

    def __init__(self, datos):
        """Inicializa el objeto."""
        obligatorias = {'SQL': str,
                      'Parametros': str,
                      'Nombre': str,
                      'Tablas': list}
        opcionales = {'Repetir_con': str}
        _Checker.__init__(self, datos, obligatorias, opcionales)


class ImagenChecker(_Checker):

    """Revisor de imagenes."""

    def __init__(self, datos):
        """Inicializa el objeto."""
        obligatorios = ['Nombre',
                       'Ruta',
                       'Celda_inicial',
                       'Celda_final']
        _Checker.__init__(self, datos, obligatorios)


class YAMLChecker(object):

    """Revisa que el archivo de control este bien conformado."""

    def __init__(self, archivo):
        """Inicializa el objeto"""
        self.__version__ = 1
        self.archivo = archivo

    def cargar(self):
        """Carga el archivo en memoria."""
        f = file(self.archivo, 'r')
        self.parsed = yaml.load(f)

    def revisar_estructura(self):
        """Revisa que el archivo este bien formado."""
        checker = DocumentoChecker(self.parsed)
        checker.check()
        for consulta in self.parsed['Consultas']:
            consulta_chk = ConsultaChecker(consulta)
            consulta_chk.check()
            for tabla in consulta['Tablas']:
                self.check_table(tabla)

        if 'Imagenes' in self.parsed:
            for imagen in self.parsed['Imagenes']:
                imagen_chk = ImagenChecker(imagen)
                imagen_chk.check()

    def check_table(self, tabla):
        """Revisa la construccion de la tabla"""
        checker = TablaChecker(tabla)
        checker.check()


if __name__ == '__main__':
    if len(sys.argv) < 2:
        raise CheckError('Se debe especicifar el archivo')

    yaml_path = os.path.abspath(sys.argv[1])
    if not os.path.isfile(yaml_path):
        raise CheckError('El archivo %s no existe' % yaml_path)
    checker = YAMLChecker(yaml_path)
    checker.cargar()
    try:
        checker.revisar_estructura()
        print yaml_path, ": OK"

    except CheckError as error:
        print yaml_path, ": ", error

exit()

