# -*- coding: utf-8 -*-
"""
Este modulo contiene las clases que interactuan directamente con COM.
Preeven los metodos para que otros objetos puedan escribir los archivos
a excel sin preoparse por los comandos COM.
"""
import math
import os


from win32com.client import constants, Dispatch


class COMWriter(object):

    """Escribe documentos de Excel usando COM."""

    def __init__(self, plantilla, archivo):
        """Inicializa el objeto.

        Args:
            plantilla -- Documento plantilla con el formato.
            archivo -- Archivo de salida o escritura.

        La plantilla es un archivo de excel que contiene la forma del reporte


        """
        patron = os.path.abspath(plantilla)
        self._archivo = os.path.abspath(archivo)
        if not os.path.exists(patron):
            raise StandardError('El archivo plantla %s no existe.' % patron)
        self._app = Dispatch("Excel.Application")
        self._app.Visible = True
        self._sheet = 1
        self._app.Workbooks.Add(patron)
        self._workgroup = self._app.ActiveWorkbook
        self._hojas = {}
        self._indexar_hojas()
        self._historial = []

    def _indexar_hojas(self):
        """Crea un diccionario con los nombres de las hojas."""
        for (i, hoja) in enumerate(self._workgroup.Sheets):
            nombre = hoja.Name
            self._hojas[nombre] = i + 1

    def escribir(self, tabla, datos, hoja=None, base=None):
        """Escribe las tablas al documento.

        Args:
            tabla -- Tabla donde insertar los datos.
            datos -- Datos a ser escritos en el archivo, debe
                     ser de la forma ((fila),(fila), ...)
            hoja -- Hoja del libro donde escribir los datos.

        """
        if len(datos) == 0:
            return
        celda = tabla.celda()
        direccion = self.rango_insercion(celda, datos)
        sheet = self._hoja_get(hoja, base)
        rango = sheet.Range(direccion)
        rango.Value = datos

        # Se debe revisar si la celda es una union de celdas, en este caso,
        # el autowrap no funciona (2007) y se debe ajustar la altura
        # manualmente
        mrango = sheet.Range(celda.celda()).MergeArea()
        if isinstance(mrango, tuple) and len(mrango[0]) > 1:
            self.arreglar_altura_union(celda.celda())

    def _hoja_get(self, hoja, base=None):
        """Devuelve la hoja del libro.

        Args :
            hoja -- Hoja a buscar.
            base -- Hoja base a usar si no se encuentra la hoja

        Ret
            Un objeto Worksheet

        Si hoja es None, se usa la hoja actual del libro.
        Si hoja es un entero, se toma como el numero de la hoja en el indice
        de hojas.
        Si hoja es texto, se asume que es el nombre de la hoja

        Si la hoja que se pide NO existe, se crea una nueva copiando la
        base y renombrandola

        """
        if hoja is None:
            hoja = self._sheet

        if isinstance(hoja, int):
            indice_hoja = hoja

        elif isinstance(hoja, str) or isinstance(hoja, unicode):
            hoja = unicode(hoja)
            if hoja in self._hojas:
                indice_hoja = self._hojas[hoja]

            else:
                hoja_obj = self._hoja_copiar(base)
                hoja_obj.Name = hoja
                self._indexar_hojas()
                return hoja_obj

        return self._app.ActiveWorkbook.Sheets(indice_hoja)

    def _hoja_copiar(self, str_base):
        """Copia una hoja.

        Args:
            str-base -- Nombre de la hoja

        Ret
            Un objeto Worksheet

        """
        base = self._hoja_get(str_base)
        base.Copy(After=base)
        self._indexar_hojas()
        hoja_nueva = self._hoja_get(base.Name + ' (2)')
        return hoja_nueva

    def rango_insercion(self, celda, datos):
        """Obtener el rango de insercion de una tabla.

        Args:
            celda -- Celda de insercion en notacion de Excel. punto.Punto
            datos -- Tabla con los datos a ser escritos en el archivo, debe
                     ser de la forma ((fila),(fila), ...)
        Ret:
            Devuelve el rango de celdas que se sera cubierto por la tabla.

        """
        delta_col = len(datos[0]) - 1
        delta_filas = len(datos) - 1
        punto_final = celda.suma(delta_col, delta_filas)

        direccion = '%s:%s' % (celda.celda(), punto_final.celda())
        return direccion

    def expandir_filas(self, celda, cantidad=1):
        """Expande la tabla.

        Expande las tablas de la plantilla para que quepan los datos que se
        van a insertar. Esto lo hace insertando una fila en 'fila' y luego
        formateando esta nueva fila con el formato de la anterior.

        Args:
            Celda -- Celda donde se deben insertar las nuevas filas.
            cantidad --Numero de filas a agregar.

        >> expandir(23, 8)

        Inserta 8 filas a partir de la fila 8, con el mismo formato de la
        fila 8.

        """
        fila = celda.fila()
        hoja = self._app.ActiveWorkbook.Sheets(self._sheet)
        for i in range(cantidad):
            rango_inicial = '%s:%s' % (fila + i, fila + i)
            rango_final = '%s:%s' % (fila + i + 1, fila + i + 1)
            rango = hoja.Range(rango_inicial)
            rango2 = hoja.Range(rango_final)
            rango2.Insert(constants.xlShiftDown)
            rango.Copy()
            rango2 = hoja.Range(rango_final)
            rango2.PasteSpecial(constants.xlPasteFormats, constants.xlNone,
                                False, False)

    def expandir_columnas(self, celda, cantidad=1):
        """Expande la tabla.

        Expande las tablas de la plantilla para que quepan los datos que se
        van a insertar. Esto lo hace insertando una columna en 'columna' y
        luego formateando esta nueva columna con el formato de la anterior.

        Args:
            celda -- Celda en la columna a insertar mas colunas.
            cantidad --Numero de filas a agregar.

        >> expandir_y(23, 8)

        Inserta 8 columnas a partir de la columna 8, con el mismo formato de la
        columna 8.

        """
        hoja = self._app.ActiveWorkbook.Sheets(self._sheet)
        for i in range(cantidad):
            celda_inicial = celda.suma(i, 0)
            celda_final = celda_inicial.suma(1, 0)
            rango_inicial = '%s:%s' % (celda_inicial.columna(),
                                       celda_inicial.columna())
            rango_final = '%s:%s' % (celda_final.columna(),
                                     celda_final.columna())
            rango = hoja.Range(rango_inicial)
            rango2 = hoja.Range(rango_final)
            rango2.Insert(constants.xlShiftToRight)
            rango.Copy()
            rango2 = hoja.Range(rango_final)
            rango2.PasteSpecial(constants.xlPasteFormats, constants.xlNone,
                                False, False)

    def guardar(self, macros):
        """Guarda el documento."""
        tipo = None
        xlOpenXMLWorkbookMacroEnabled = 52
        if os.path.exists(self._archivo):
            os.remove(self._archivo)

        if macros == 1:
            tipo = xlOpenXMLWorkbookMacroEnabled

        if tipo is None:
            self._app.ActiveWorkbook.SaveAs(self._archivo)

        else:
            self._app.ActiveWorkbook.SaveAs(self._archivo, tipo)

        self._app.ActiveWorkbook.Close()
        self._app.Visible = True

        print 'Guardando %s' % self._archivo

    def arreglar_altura_union(self, direccion):
        """Arregla la altura de las celdas unidas para que quepa el texto.

        Cuando se escribe en una celda que ha sido unida con otras, la funcion
        'ajustar texto' no funciona, esta solo funciona para celdas
        individuales. Debido a esto hay que arreglar la altura de la celda
        manualmente.

        Args:
            direccion -- Direccion de la celda a corregir ej: 'B14'

        """
        alto_linea = 15
        celda = self._app.ActiveWorkbook.Sheets(self._sheet).Range(direccion)
        merged_area = celda.MergeArea
        ancho_original = merged_area.Width
        merged_area.UnMerge()
        ancho_celda = celda.ColumnWidth
        alto_original = celda.RowHeight
        celda.RowHeight = 15
        celda.Columns.AutoFit()

        ancho_final = celda.Width
        lineas = math.ceil(ancho_final / ancho_original + 0.1)
        if alto_original < (lineas * alto_linea):
            celda.RowHeight = lineas * alto_linea
        else:
            celda.RowHeight = alto_original
        merged_area.Merge()
        merged_area.WrapText = True
        celda.ColumnWidth = ancho_celda

    def insertar_foto(self, celda_ini, celda_fin, ruta):
        """Inserta una foto en el documento."""
        hoja = self._app.ActiveWorkbook.Sheets(self._sheet)
        left = hoja.Range(celda_ini).Left
        top = hoja.Range(celda_ini).Top
        width = hoja.Range(celda_fin).Left - left
        height = hoja.Range(celda_fin).Top - top
        hoja.Shapes.AddPicture(os.path.normpath(ruta), False, True, left, top,
                                width, height)
