# -*- coding: utf-8 -*-
import datetime
import os
from punto import Celda


class Transformadas(object):

    """Define una transformacion de las tablas.

    Este objeto tiene una pila de funciones a aplicar en los datos junto
    con los parametros necesarios para ejecutarlas.
    """

    def __init__(self):
        """Inicializa el objeto"""
        self._pila = []
        self.funciones = {'identidad': self._identidad,
                 'columna': self._columna,
                 'tabular_columnas': self._tabular_columnas,
                 'completar_fechas': self._completar_fechas,
                 'dividir_en_columnas': self._dividir_en_columnas,
                 'filtrar_columnas': self._filtrar_columnas,
                 'filtrar_columnas_impares': self._filtrar_columnas_impares,
                 'filtrar_columnas_pares': self._filtrar_columnas_pares,
                 'totalizar_columnas': self._totalizar_filas,
                 'imagen': self._imagen,
                 'dato': self._dato}

    def pila(self):
        """Obtenern la pila de funciones."""
        return self._pila

    def funcion_agregar(self, str_funcion, argumentos):
        """Agrega una funcion a la pila de la transformada."""
        try:
            funcion = self.funciones[str_funcion]

        except KeyError as error:
            raise StandardError('La funcion %s no existe' % error)

        self._pila.append((funcion, argumentos))

    def _dato(self, tabla, fila, columna):
        """Devuelve el dato de una fila y columna especificadas."""
        fila = int(fila)
        columna = int(columna)
        return tabla[fila][columna]

    def _columna(self, tabla, columna):
        """Devuelve una columna de una tabla.

        Args:
            tabla -- Tabla de datos ((fila), (fila), ...)
            columna -- Columna.

        Ret:
            ((f1), (f2),..., (fn))

            Donde f1, f2, ...fn son los valores de los elemento de la
            columna en la fila n.

        Ej:
            >>> tabla = ((1, 2, 3), (4, 5, 6), (7, 8, 9))
            >>> _columna(tabla, '2')
            [(3,), (6,), (9,)]

        """
        tabla_modificada = [(fila[int(columna)], ) for fila in tabla]
        return tabla_modificada

    def _tabular_columnas(self, tabla, *parametros):
        """Inserta valores None entre las columnas para corregir la alineacion.

        Args:
            tabla -- Tabla de datos ((fila), (fila), ...)
            parametros -- Numeros de las columnas que contendran los datos.

        Ret:
            ((fila1), (fila2),..., (filan))

        En algunos casos, se necesita agregar columnas vacias a las tablas de
        datos antes de escribirlas, esto sucede si el formato tiene columnas
        vacias o si se tienen celdas unidas (merged).

        Ej:
            >>> tabla = ((1, 2, 3), (4, 5, 6), (7, 8, 9))
            >>> _tabular_columnas(tabla, '1', '3', '4')
            [[1, None, 2, 3], [4, None, 5, 6], [7, None, 8, 9]]

        """
        columnas = map(int, parametros)
        ultima_columna = max(columnas)
        tabla_modificada = []
        correspondencia = tuple(enumerate(columnas))
        try:
            for fila in tabla:
                nueva_fila = [None] * ultima_columna
                for (columna_original, columna_nueva) in correspondencia:
                    nueva_fila[columna_nueva - 1] = fila[columna_original]

                tabla_modificada.append(nueva_fila)
        except IndexError:
            print 'Se intento acceder datos en la matriz que no existen'

        return tabla_modificada

    def _identidad(self, tabla, *parametros):
        """Devuelve la misma tabla sin modificarla."""
        return tabla

    def _dividir_en_columnas(self, tabla, *columna):
        """Divide una tabla en columnas.

        Args:
            tabla --Tabla de datos ((fila), (fila), ...)
            columna -- Columna que se debe usar como criterio para la separación.

        Ret:
            ((fila1), (fila2),..., (filan))

        Divide la tabla en columnas usando como criterio el cambio de valor
        de una columna.

        Ej:

        >>> tabla = ((1, 2, 3), (4, 2, 6), (4, 3, 6), (4, 3, 4), \
                    (7, 8, 9), (1, 8, 1))
        >>> _dividir_en_columnas(tabla, '2')
        ((1, 2, 3, 4, 3, 6, 7, 8, 9), (4, 2, 6, 4, 3, 4, 1, 8, 1))

           tabla                     return
        ((1, 2, 3)       ((1, 2, 3, 4, 3, 6, 7, 8, 9)
         (4, 2, 6)        (4, 2, 6, 4, 3, 4, 1, 8, 1))
         (4, 3, 6)   =>
         (4, 3, 4)
         (7, 8, 9)
         (1, 8, 1))

        """
        columna = int(columna[0])
        tabla_modificada = [[]]
        indice = 0
        indice_actual = None
        for fila in tabla:
            if indice_actual != fila[columna]:
                indice = 0
                tabla_modificada[indice] = tabla_modificada[indice] + list(fila)
                indice_actual = fila[columna]

            else:
                try:
                    tabla_modificada[indice] = tabla_modificada[indice] + list(fila)

                except:
                    tabla_modificada.append(list(fila))

            indice += 1

        return tabla_modificada

    def _filtrar_columnas(self, tabla, *parametros):
        """Devuelve la tabla con las columnas seleccionadas.

        Args:
            tabla -- Tabla con los datos. ((fila1) (fila2) (fila3))
            parametros -- Columnas a mantener. ('columna1', 'columna2')

        Ret:
            ((fila1), (fila2),..., (filan))

        Devuelve una tabla que contiene solo las columnas seleccionadas de la
        tabla original.

        Ej:
            >>> tabla = ((1, 2, 3), (4, 2, 6), (4, 3, 6), (4, 3, 4), (7, 8, 9))
            >>> _filtra_columnas(tabla, '1', '2')
            ((1, 2), (4, 2), (4, 3), (4, 3), (7, 8))

        """
        indices = map(int, parametros)
        tabla_modificada = []
        for fila in tabla:
            nueva_fila = []
            for indice in indices:
                nueva_fila.append(fila[indice])

            tabla_modificada.append(nueva_fila)

        return tabla_modificada

    def _filtrar_columnas_impares(self, tabla, *parametros):
        """Filtrar las columnas impares."""
        i = range(0, len(tabla[0]), 2)
        return self._filtrar_columnas(tabla, *i)

    def _filtrar_columnas_pares(self, tabla, *parametros):
        """Filtrar las columnas impares."""
        i = range(1, len(tabla[0]), 2)
        return self._filtrar_columnas(tabla, *i)

    def _totalizar_filas(self, tabla, *parametros):
        """Agrega una columna con las formulas que totalizan las filas.

        Args:
            tabla -- Tabla con los datos. ((fila1) (fila2) (fila3))
                iniciar a totalizar.
            parametros -- Celda donde inicia la tabla.

        Ret:
            ((fila1), (fila2),..., (filan))

        Algunas tablas deben ser totalizadas en sentido horizontal, es decir
        por filas. Esta funcion agrega una columna al final de la tabla que
        contiene las formulas necesarias para totalizar las filas. El parametro
        parametro contiene la celda inicial donde se va a insertar la tabla,
        este parametro es necesario pues esta funcion no totaliza directamente
        las tablas y debe conocer la celda inical para determinar las formulas
        necesarias.

        Ej:
            >>> tabla = ((1, 2), (4, 2), (4, 3))
            >>> _totalizar_columnas(tabla, 'A1')
            ((1, 2, '=SUM(A1:B1)), (4, 2, '=SUM(A2:B2)), (4, 3, '=SUM(A3:B3)))

        """
        n_columnas = len(tabla[0]) - 1   # Fence posting
        tabla_modificada = []
        contador = 0
        for fila in tabla:
            punto_inicial = Celda(parametros[0]).suma(0, contador)
            punto_final = punto_inicial.suma(n_columnas, 0)
            suma = '=SUM(%s:%s)' % (punto_inicial.celda(), punto_final.celda())
            fila_final = fila + [suma]
            tabla_modificada.append(fila_final)
            contador += 1
        return tabla_modificada

    def _imagen(self, tabla, *parametros):
        """Agrega una imagen en la celda especificada."""
        celda_fin = parametros[0]
        ruta = tabla[0][0]
        return (celda_fin, os.path.abspath(ruta))

    def _completar_fechas(self, tabla, *parametros):
        """Completa las tablas con valores faltantes.

        Args:
            tabla -- Tabla original con los resultados de la consulta.
            *parametros -- Valores adicionales para la funcion
                parametros[0] -- Indice de la columna con las fechas a
                                 completar. Las fechas deben ser del tipo
                                 datetime.date
                parametros[1] -- Valor con el cual completar los datos
                parametros[2] -- Fecha inicial
                parametros[3] -- Numero de filas a agregar

        Ret:
            Tabla con las filas completas con todo el conjunto de fechas
            intermedias.

        Generalmente cuando se hace consultas a una base de datos por
        resultados por fechas, no todas las fechas tienen resultados, y
        existen dias especificos en los cuales no hay datos; esta función
        completa las tablas de resultados incluyendo resultados nulos en los
        dias que no aparecen en la tabla.

        Ej:
            >>> tabla = ((2012-01-02, 2), ('2012-01-05, 9))
            >>> _completar_fechas(tabla, 0, None)
            ((2012-01-01, None), (2012-01-02, 2), (2012-01-03, None),
             (2012-01-04, None), (2012-01-05, 9))

        Las fechas *deben* ser datetime.date, en el ejemplo se muestra su
        representacion como cadena

        """
        if len(tabla) == 0:
            return tabla
        n_filas = len(tabla[0])
        indice = int(parametros[0])
        fill_with = [parametros[1]] * (n_filas - 1)
        try:
            1 / parametros[2]
            fecha_inicial = datetime.date(tabla[0][indice].year,
                                          tabla[0][indice].month,
                                          1)
        except:
            fecha_inicial = tabla[0][indice]

        try:
            cantidad = int(parametros[3])
        except:
            cantidad = 1

        tabla_modificada = []
        # Fila inicial
        if tabla[0][indice] != fecha_inicial:
            for c in range(cantidad):
                tabla_modificada.append([fecha_inicial] + fill_with)

        for (e, fila) in enumerate(tabla):
            fecha_actual = fila[indice]
            try:
                n_dias = (fecha_actual - fecha_inicial).days
            except:
                print tabla
            for i in range(1, n_dias):
                a_sumar = datetime.timedelta(i)
                fila_vacia = [fecha_inicial + a_sumar] + fill_with
                for c in range(cantidad):
                    tabla_modificada.append(fila_vacia)

            tabla_modificada.append(fila)
            fecha_inicial = fecha_actual

        return tabla_modificada

