#!/usr/bin/python
"""
Este modulo contiene las clases encargadas de interpretar los archivos de
cotrol. El interprete principal es YAMLParser
"""
import yaml
import re

import punto
import xls_formato
from tabla import Tabla
from consulta import Consulta
from transformada import Transformadas
from imagen import Imagen


class ParserException (Exception):

    """Excepcion del interprete de archivos de control."""

    def __init__(self, valor):
        self.value = valor

    def __str__(self):
        return repr(self.value)


class YAMLParser():

    """Interpretador de archivos de control escritos en YAML."""

    def __init__(self):
        """Inicializa el objeto."""
        self._version = 1
        self._debug = False
        self._regex_numero = re.compile(r"^[-+]?[0-9]*\.?[0-9]+$")
        self._regex_variable = re.compile(r"^[A-Za-z]+[A-Za-z0-9_]*")
        self._regex_celda = re.compile(r"""['"][A-Za-z]+[0-9]+['"]""")

    def parse(self, archivo, solo_encabezado=False):
        """Interpreta un archivo de control escrito en YAML."""
        formato =  xls_formato.XlsFormato()
        f = file(archivo, 'r')
        parsed = yaml.load(f)
        f.close()
        formato.nombre_set(parsed['Nombre'])
        formato.descripcion_set(parsed['Descripcion'])
        formato.version_set(parsed['Version_minima'])
        formato.autor_set(parsed['Autor'])
        formato.email_set(parsed['Email'])
        formato.edicion_set(parsed['Fecha_edicion'])
        formato.plantilla_set(parsed['Plantilla'])
        formato.macros_set(parsed['Macros'])
        formato.archivo_salida_set(parsed['Archivo_salida'])
        formato.datos_usuario_set(parsed['Datos'])
        if solo_encabezado:
            return formato

        for consulta_data in parsed['Consultas']:
            consulta = self.parse_consulta(consulta_data)
            formato.consulta_agregar(consulta)

        if 'Imagenes' in parsed:
            for imagen_data in parsed['Imagenes']:
                imagen = self.parse_imagenes(imagen_data)
                formato.imagen_agregar(imagen)

        return formato

    def parse_imagenes(self, imagen_data):
        """Interpretar las imagenes."""
        imagen = Imagen()
        try:
            imagen.nombre = imagen_data['Nombre']
            imagen.ruta = imagen_data['Ruta']
            imagen.celda_inicial = imagen_data['Celda_inicial']
            imagen.celda_final = imagen_data['Celda_final']

        except KeyError as error:
            raise ParserException("La opcion %s es obligatoria para imagenes" % error)

        return imagen

    def parse_consulta(self, consulta_data):
        """Interpreta las consultas."""
        if self._debug:
            print 'Parsing %s' % consulta_data['Nombre']
        consulta = Consulta(nombre=consulta_data['Nombre'],
                            query=consulta_data['SQL'],
                            parametros=consulta_data['Parametros'])
        if 'Repetir_con' in consulta_data:
                consulta.repetir_con_set(consulta_data['Repetir_con'])

        for tabla_data in consulta_data['Tablas']:
            tabla = Tabla(tabla_data['Nombre'])
            if 'Celda' in tabla_data:
                tabla.celda_set(tabla_data['Celda'])
            elif 'Guardar_en' in tabla_data:
                tabla.guardar_en_set(tabla_data['Guardar_en'])
            else:
                raise StandarError('La tabla no tiene celda o variable asignada')

            if 'Tipo' in tabla_data:
                tabla.tipo_set(tabla_data['Tipo'])
            if 'Hoja' in tabla_data:
                tabla.hoja_set(tabla_data['Hoja'])
            if 'Hoja_Base' in tabla_data:
                tabla.hoja_base_set(tabla_data['Hoja_Base'])
            if 'Expandir_filas' in tabla_data:
                tabla.expandir_filas_set(tabla_data['Expandir_filas'])
            if 'Expandir_columnas' in tabla_data:
                tabla.expandir_columnas_set(tabla_data['Expandir_columnas'])

            transformadas = Transformadas()
            tabla.transformada_set(transformadas)
            for funcion_data in tabla_data['Transformada']:
                (funcion, parametros) = self.parse_transformada(funcion_data)
                transformadas.funcion_agregar(funcion, parametros)

            consulta.tabla_agregar(tabla)

        return consulta

    def parse_transformada(self, texto):
        """Interpretar la transformadas de un punto."""
        tokens = texto.split(',')
        if tokens == texto and tokens != texto.split(' '):
            raise ParserException('No se encontraron parametros: "%s"' % texto)
        funcion = self.parse_funcion(tokens[0])
        tokens.remove(tokens[0])
        argumentos = self.parse_funcion_argumentos(tokens[0])
        return (funcion, argumentos)

    def parse_funcion(self, texto):
        """Interpreta una funcion."""
        funcion = texto.strip()
        match = self._regex_variable.match(funcion)
        if match is None:
            raise ParserException('Nombre de funcion erroneo: "%s"' % funcion)
        return funcion

    def parse_funcion_argumentos(self, texto):
        """Interpreta los argumentos de las funciones."""
        tokens = texto.strip().split(' ')
        argumentos = []
        for token in tokens:
            argumento = token.strip()
            if self._regex_numero.match(argumento) is None and \
                    self._regex_variable.match(argumento) is None and\
                    self._regex_celda.match(argumento) is None:
                raise ParserException('Nombre de argumento erroneo: "%s"' % argumento)

            argumentos.append(argumento)

        return argumentos
